import uuid
from django.db import models
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django_resaas.core.base.models import TimeModel


def documento_path(instance, file_name):
    return f'{instance.tipo_entidade.nome}/{instance.nome}/{file_name}'

class TipoDocumento(TimeModel):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    nome = models.CharField(max_length=200)
    detalhes = models.CharField(max_length=200)

    def __str__(self):
        return self.nome

class Documento(TimeModel):

    tipo = models.ForeignKey(TipoDocumento, on_delete=models.CASCADE)
    numero = models.CharField(max_length=100)

    data_emissao = models.DateField(null=True, blank=True)
    data_validade = models.DateField(null=True, blank=True)

    arquivo = models.FileField(upload_to=documento_path, null=True, blank=True)

    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.UUIDField()
    content_object = GenericForeignKey('content_type', 'object_id')

    class Meta:
        unique_together = ('tipo', 'numero')

    def __str__(self):
        return f"{self.tipo.nome} - {self.numero}"



from rest_framework import serializers

from django_resaas.core.base.serializers import BaseSerializer
from django_resaas.models.user import User


class UserSerializer(BaseSerializer):
    
    class Meta:
        model = User
        fields = ['id', 'email', 'username', 'perfil', 'mobile']


from django.apps import AppConfig
class DjangoResaasConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = "django_resaas"
    verbose_name = "Django SaaS"

    def ready(self):
        self.load_permissions()
        # self.load_admin()

    def load_permissions(self):
        import django_resaas.core.signals.permissions

    def load_admin(self):
        from django_resaas.core.base.mixins.admin.auto import register_all_models
        register_all_models()


from django.db import transaction
from django.contrib.auth.models import Group
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
from rest_framework.permissions import IsAuthenticated

from django_resaas.models.tipo_entidade import TipoEntidade
from django_resaas.models.entidade import Entidade
from django_resaas.models.sucursal import Sucursal
from django_resaas.models.entidade_user import EntidadeUser
from django_resaas.models.sucursal_user import SucursalUser
from django_resaas.models.sucursal_user_group import SucursalUserGroup

from django_resaas.core.utils.translate import Translate


class TenantAPIView(APIView):
    """
    Bootstrap inicial:
    TipoEntidade → Entidade → Sucursal → Grupo
    Tudo associado a um utilizador.
    """
    permission_classes = [IsAuthenticated]

    # @transaction.atomic
    def get(self, request):
        user = request.user

        data = {
            "tipo_entidade": "Saas",
            "entidade": "Mytech",
            "sucursal": "Sede",
            "grupo": "Admin",
        }

        # ------------------------
        # 1. TipoEntidade
        # ------------------------
        tipo_entidade, _ = TipoEntidade.objects.get_or_create(
            nome=data["tipo_entidade"],
            defaults={"estado": 1}
        )

        # ------------------------
        # 2. Entidade
        # ------------------------
        entidade, created_entidade = Entidade.objects.get_or_create(
            nome=data["entidade"],
            tipo_entidade=tipo_entidade
        )

        # ManyToMany → DEPOIS
        entidade.admins.add(user)

        EntidadeUser.objects.get_or_create(
            user=user,
            entidade=entidade
        )

        # ------------------------
        # 3. Sucursal
        # ------------------------
        sucursal, _ = Sucursal.objects.get_or_create(
            nome=data["sucursal"],
            entidade=entidade
        )

        SucursalUser.objects.get_or_create(
            user=user,
            sucursal=sucursal
        )

        # ------------------------
        # 4. Grupo
        # ------------------------
        grupo, _ = Group.objects.get_or_create(
            name=data["grupo"]
        )

        SucursalUserGroup.objects.get_or_create(
            user=user,
            sucursal=sucursal,
            group=grupo
        )

        user.groups.add(grupo)

        # ------------------------
        # RESPONSE
        # ------------------------
        return Response(
            {
                "alert_success": Translate.tdc(
                    request,
                    "Configuração inicial criada com sucesso"
                ),
                "data": {
                    "tipo_entidade": tipo_entidade.nome,
                    "entidade": entidade.nome,
                    "sucursal": sucursal.nome,
                    "grupo": grupo.name,
                    "user": user.username,
                }
            },
            status=status.HTTP_201_CREATED
        )



import uuid

from django.db import models

from django_resaas.models.modulo import Modulo
from django_resaas.models.entidade import Entidade
from django_resaas.core.base.models import TimeModel
from django.contrib.contenttypes.models import ContentType


class EntidadeModelo(TimeModel):
    entidade = models.ForeignKey(Entidade, on_delete=models.CASCADE)
    modelo = models.ForeignKey(ContentType, on_delete=models.CASCADE)

    class Meta:
        permissions = ()

    def __str__(self):
        return f'{self.entidade.nome} | {self.modelo.nome}'



# =========================
# Django REST Framework
# =========================
from rest_framework import filters
from rest_framework import status
from rest_framework import viewsets
from django_resaas.models.pessoa import Pessoa

from django_resaas.data.pessoa.serializers.pessoa import PessoaSerializer

class  pessoaAPIView(viewsets.ModelViewSet):

    filter_backends = (filters.SearchFilter,)
    
    serializer_class = PessoaSerializer
    queryset = Pessoa.objects.all()
    lookup_field = "id"

    def get_queryset(self):
        return self.queryset.filter().order_by('-id')